"""
    nllegalcit/errors.py

    Copyright 2023, Martijn Staal <nllegalcit [at] martijn-staal.nl>

    Available under the EUPL-1.2, or, at your option, any later version.

    SPDX-License-Identifier: EUPL-1.2
"""


class CitationParseException(Exception):
    """Something went wrong parsing a citation"""
